import os
import configparser
import errno

from appdirs import user_config_dir, user_data_dir
from rich.console import Console


APP_NAME = "jotpad"
APP_AUTHOR = "jotpad"

class Config:
    def __init__(self):
        self._config = configparser.ConfigParser()
        self._config.read(os.path.join(f"{user_config_dir(APP_NAME)}", "config.ini"))

        if "jotpad" not in self._config:
            self.init()
    
    def _write(self):
        with open(os.path.join(f"{user_config_dir(APP_NAME)}", "config.ini"), "w") as f:
            self._config.write(f)

    def init(self):
        try:
            os.makedirs(user_config_dir(APP_NAME))
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise RuntimeError("Unable to create config directory")

        if os.name == 'nt':
            preffered = "notepad"
        else:
            preffered = "vi"

        home = input("Enter path to notes directory (e.g. /home/user/notes): ")
        editor = input(f"Enter preffered text editor (default {preffered}): ")
        extension = input("Enter default file extension (default txt): ")
        editor = editor if editor else preffered
        extension = extension if extension else "txt"
        
        self._config["jotpad"] = {
            "home": home,
            "editor": editor,
            "default_extension": extension,
        }
        with open(os.path.join(f"{user_config_dir(APP_NAME)}", "config.ini"), "w") as f:
            self._config.write(f)
        
        # console = Console()
        # console.print(f"[DEBUG] full config file path = {os.path.join(user_config_dir(APP_NAME), 'config.ini')}")
        # console.print(f"[DEBUG] config_dir={user_config_dir(APP_NAME)}")
        # console.print(f"[DEBUG] data_dir={user_data_dir(APP_NAME, APP_AUTHOR)}")
        # console.print(f"[DEBUG] config.home={self._config['jotpad']['home']}")
        # console.print(f"[DEBUG] config.editor={self._config['jotpad']['editor']}")
        # console.print(f"[DEBUG] config.default_extension={self._config['jotpad']['default_extension']}")
    
    @property
    def home(self):
        if "home" in self._config["jotpad"]:
            return self._config["jotpad"]["home"]
        
        try:
            os.makedirs(user_data_dir(APP_NAME, APP_AUTHOR))
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise RuntimeError("Unable to create data directory")
        return user_data_dir(APP_NAME, APP_AUTHOR)

    @home.setter
    def home(self, value):
        self._config["jotpad"]["home"] = value
        self._write()
    
    @property
    def editor(self):
        return self._config["jotpad"]["editor"]

    @editor.setter
    def editor(self, value):
        self._config["jotpad"]["editor"] = value
        self._write()

    @property
    def default_extension(self):
        return self._config["jotpad"]["default_extension"]
    
    @default_extension.setter
    def default_extension(self, value):
        self._config["jotpad"]["default_extension"] = value
        self._write()



import os
import sys

import fileinput
import functools
from pathlib import Path
from typing import Optional
import time

import typer
from rich.table import Table
from rich.console import Console
from rich.markdown import Markdown

from .config import Config


app = typer.Typer()
APP_NAME = "jotpad"
APP_AUTHOR = "jotpad"


def get_paths(config):
    return sorted(Path(config.home).iterdir(), key=os.path.getmtime, reverse=True)


@app.callback(invoke_without_command=True)
def default(
    ctx: typer.Context,
    n: int = typer.Option(1, "--note-index", "-n"),
    dump: bool = typer.Option(False, "--dump", "-d"),
):  
    if not ctx.invoked_subcommand:
        config = Config()
        console = Console()
        paths = get_paths(config)

        if not len(paths):
            name = f"note-{len(paths) + 1}.{config.default_extension}"
            note_path = os.path.join(f"{config.home}", name)

            with open(note_path, "w") as f:
                f.write(content)
        
        else:
            note_path = paths[n - 1]

        if not os.isatty(0):
            content = functools.reduce(lambda a, b: a + b, fileinput.input(files=()))

            with open(note_path, "a") as f:
                f.write(f"\n{content}")

        if dump:
            with open(note_path, "r") as f:
                note = f.read()
            console.print(note)
        else:
            os.system(f"{config.editor} {note_path}")


@app.command()
def new(
    dump: bool = typer.Option(False, "--dump", "-d"),
):
    console = Console()
    config = Config()
    name = f"note-{len(get_paths(config)) + 1}.{config.default_extension}"
    note_path = os.path.join(f"{config.home}", name)

    if not os.isatty(0):
        content = functools.reduce(lambda a, b: a + b, fileinput.input(files=()))
    else:
        content = ""
    
    with open(note_path, "w") as f:
        f.write(content)

    if dump:
        with open(note_path, "r") as f:
            note = f.read()
        console.print(note)
    else:
        os.system(f"{config.editor} {note_path}")


@app.command()
def rm(
    ctx: typer.Context,
    n: int,
):  
    if not ctx.invoked_subcommand:
        config = Config()
        console = Console()
        paths = get_paths(config)
        note_path = paths[n - 1]
        os.remove(note_path)


@app.command()
def ls():
    console = Console()
    config = Config()
    paths = get_paths(config)

    table = Table(title="Notes")
    table.add_column("index", justify="right")
    table.add_column("modified")
    table.add_column("content")

    with console.pager():
        for i, p in enumerate(paths):
            modified = time.ctime(os.path.getmtime(p))

            with open(p, "r") as f:
                note = f.read()
                first = note.split("\n")[0]
                first = first if len(first) < 90 else f"{first[:87]}..."
            table.add_row(f"{i + 1}", modified,  f"{first}")
        console.print(table)


@app.command()
def config(
    editor: Optional[str] = typer.Option(None, "--editor", "-e"),
    home: Optional[str] = typer.Option(None, "--home", "-h"),
    init: Optional[bool] = typer.Option(False, "--init", "-i"),
    default_extension: Optional[str] = typer.Option(None, "--ext", "-x"),
):
    config = Config()
    console = Console()

    if init:
        config.init()
    if editor:
        config.editor = editor
    if home:
        config.home = home
    if default_extension:
        config.default_extension = default_extension

    table = Table(title="Config")
    table.add_column("key", justify="right")
    table.add_column("value")
    table.add_row("home", config.home)
    table.add_row("editor", config.editor)
    table.add_row("default_extension", config.default_extension)
    console.print(table)


if __name__ == "__main__":
    app()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="jotpad",
    version="0.0.14-alpha",
    install_requires=[
        "appdirs>=1.4.0",
        "typer[all]>=0.6.0",
    ],
    author="Scott Russell",
    author_email="me@scottrussell.net",
    description="Jotpad is a cli tool for managing notes",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/scrussell24/jotpad",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    entry_points = '''
        [console_scripts]
        jot=jotpad:app
    '''
)
